from earthdiagnostics import Utils, cdftools
from earthdiagnostics.models import Grid
from autosubmit.config.log import Log
from nco import NCOException
import os
import shutil


class Circulation(object):

    @staticmethod
    def moc(input_file, output_file):
        """
        Compute the MOC for oceanic basins
        Created in March 2012
        Author : vguemas@ic3.cat

        :param input_file:  input grid_V file name
        :type input_file: str
        :param output_file: output file name (=> 2D, depth-y)
        :param output_file: str
        :return:
        """
        nco = Utils.nco
        if os.path.exists(output_file):
            os.remove(output_file)

        cdftools.run('cdfmoc', input=input_file, output=output_file)
        nco.ncwa(input=output_file, output=output_file, options='-O -a x')
        nco.ncks(input=output_file, output=output_file, options='-O -x -v nav_lon,nav_lat')

        Utils.rename_variable(output_file, 'time_counter', 'time', False, True)
        Utils.rename_variable(output_file, 'gsize', 'y', False)

        temp = Utils.get_temp_file()
        shutil.copy(input_file, temp)
        Utils.rename_variable(output_file, 'time_counter', 'time', False, True)

        nco.ncks(input=input_file, output=output_file, options='-A -v nav_lon,nav_lat')

    @staticmethod
    def area_moc(input_file, lat_min, lat_max, output_file, depth_min=1000, depth_max=2000, basin='zomsfatl'):
        """
        Compute an Atlantic MOC index by averaging the meridional overturning
        in a latitude band between 1km and 2km
        or any other index averaging the meridional overturning in
        a given basin and a given domain

        Created in March 2012         Author : vguemas@ic3.cat

        :param input_file: input moc file name
        :param lat_min: latitude min
        :param lat_max: latitude max
        :param output_file: output file name      ( => index )
        :param depth_min: depth min (default : 1km)
        :param depth_max: depth max (default : 2km)
        :param basin: basin (default :  zomsfatl)
        :return:
        """
        nco = Utils.nco
        cdo = Utils.cdo
        temp = Utils.get_temp_file()
        temp2 = Utils.get_temp_file()

        handler = Utils.cdo.openCdf(input_file)
        if 'x' in handler.dimensions:
            nco.ncwa(input=input_file, output=temp, options='-O -a x')
        else:
            shutil.copy(input_file, temp)
        handler.close()

        nco.ncrename(input=temp, output=temp2, options='-v nav_lat,lat -d y,lat')

        nco.ncks(input=temp2, output=temp, options='-O -v {0},time,depthw,lat'.format(basin))
        nco.ncks(input=temp, output=temp, options='-O -d lat,{0:.1f},{1:.1f} -d depthw,'
                                                  '-{2:.1f},-{3:.1f}'.format(lat_min, lat_max, depth_max, depth_min))

        cdo.vertmean(input=temp, output=temp2)
        nco.ncap2(input=temp2, output=temp, options='-O -s "coslat[lat]=cos(lat[lat]*3.141592657/180.0)"')
        nco.ncwa(input=temp, output=temp, options='-w coslat -a lat')
        nco.ncks(input=temp, output=output_file, options='-O -v {0},time'.format(basin))

    @staticmethod
    def max_moc(input_file, lat_min, lat_max, output_file, depth_min, depth_max):
        """
        Compute an Atlantic MOC index by finding the maximum  of the annual
        mean meridional overturning in a latitude / depth region

        Created in March 2012         Author : vguemas@ic3.cat

        :param input_file: input moc file name
        :param lat_min: latitude min
        :param lat_max: latitude max
        :param output_file: output file name
        :param depth_min: depth mean
        :param depth_max: depth max
        :return:
        """

        nco = Utils.nco
        cdo = Utils.cdo
        temp = Utils.get_temp_file()
        temp2 = Utils.get_temp_file()

        handler = Utils.cdo.openCdf(input_file)
        if 'x' in handler.dimensions:
            nco.ncwa(input=input_file, output=temp, options='-O -a x')
        else:
            shutil.copy(input_file, temp)
        handler.close()

        Utils.rename_variable(temp, 'record', 'x')
        nco.ncdpq(input=temp, output=temp, options='-O -h -a time,x')
        nco.ncdpq(input=temp, output=temp, options='-O -h -a depthw,x')
        nco.ncdpq(input=temp, output=temp, options='-O -h -a y,x')

        cdo.yearmean(input=temp, output=temp2)

        cdftools.run('cdfmaxmoc', temp2, output=output_file, options='{0:.1f} {1:.1f} '
                                                                     '{2:.1f} {3:.1f}'.format(lat_min, lat_max,
                                                                                              depth_min, depth_max))
        Utils.rename_variable(output_file, 'time_counter', 'time', False, True)

    @staticmethod
    def psi(input_file_u, input_file_v, output_file):
        """
        Compute the barotropic stream function

        Created in March 2012         Author : vguemas@ic3.cat
        z
        :param input_file_u: input grid_U file name
        :rtype input_file_U: str
        :param input_file_v: input grid_V file name
        :rtype input_file_V: str
        :param output_file: output file name without nc extension  (=> 2D x-y)
        :rtype output_file: str
        """
        integrated_u = Utils.get_temp_file()
        integrated_v = Utils.get_temp_file()
        cdftools.run('cdfpsi', input=[input_file_u, input_file_v], output=integrated_u, )
        cdftools.run('cdfpsi', input=[input_file_u, input_file_v], output=integrated_v, options='V')

        Utils.nco.ncea(input=[integrated_u, integrated_v], output=output_file)
        Utils.rename_variable(output_file, 'time_counter', 'time', False, True)

    # noinspection PyPep8Naming
    @staticmethod
    def gyres(input_psi, input_grid, output_file):
        """
        Compute the intensity of the subtropical and subpolar gyres

        Created in October 2013         Author : vguemas@ic3.cat

        :param input_psi: input psi file name
        :param input_grid: input grid
        :param output_file: output file name   ( => index )
        :return:
        """

        if input_grid in [Grid.ECEARTH_2_3_O1L42, Grid.ECEARTH_3_0_O1L46,
                          Grid.NEMO_3_2_O1L42, Grid.NEMO_3_3_O1L46,
                          Grid.NEMOVAR_O1L42]:
            subpolNAtl = [230, 275, 215, 245]
            subpolNPac = [70, 145, 195, 235]
            subtropNPac = [45, 175, 165, 220]
            subtropSPac = [195, 275, 175, 225]
            subtropNAtl = [230, 275, 215, 245]
            subtropSAtl = [70, 145, 195, 235]
            subtropInd = [45, 175, 165, 220]
            ACC = [195, 275, 175, 225]

        elif input_grid in [Grid.ECEARTH_3_0_O25L46, Grid.ECEARTH_3_0_O25L75,
                            Grid.GLORYS2_V1_O25L75]:
            raise Exception("Option gyres not available yet for {0}".format(input_grid))
        else:
            raise Exception("Input grid {0} not recognized".format(input_grid))

        temp = Utils.get_temp_file()
        Circulation.gyre(input_psi, subpolNAtl, output_file,  invert=True)
        Utils.rename_variable(output_file, 'sobarstf', 'subpolNAtl')

        Circulation.gyre(input_psi, subpolNPac, temp, invert=True)
        Utils.rename_variable(temp, 'sobarstf', 'subpolNPac')
        Utils.nco.ncks(input=temp, output=output_file, options='-A')

        Circulation.gyre(input_psi, subtropNPac, temp)
        Utils.rename_variable(temp, 'sobarstf', 'subtropNPac')
        Utils.nco.ncks(input=temp, output=output_file, options='-A')

        Circulation.gyre(input_psi, subtropSPac, temp)
        Utils.rename_variable(temp, 'sobarstf', 'subtropSPac')
        Utils.nco.ncks(input=temp, output=output_file, options='-A')

        Circulation.gyre(input_psi, subtropNAtl, temp)
        Utils.rename_variable(temp, 'sobarstf', 'subtropNAtl')
        Utils.nco.ncks(input=temp, output=output_file, options='-A')

        Circulation.gyre(input_psi, subtropSAtl, temp)
        Utils.rename_variable(temp, 'sobarstf', 'subtropSAtl')
        Utils.nco.ncks(input=temp, output=output_file, options='-A')

        Circulation.gyre(input_psi, subtropInd, temp)
        Utils.rename_variable(temp, 'sobarstf', 'subtropInd')
        Utils.nco.ncks(input=temp, output=output_file, options='-A')

        Circulation.gyre(input_psi, ACC, temp)
        Utils.rename_variable(temp, 'sobarstf', 'ACC')
        Utils.nco.ncks(input=temp, output=output_file, options='-A')

    @staticmethod
    def gyre(input_file, site, output_file, invert=False):
        """
        Compute the intensity of a given gyre

        Created in October 2013         Author : vguemas@ic3.cat

        :param input_file: input oce file name containing sobarstf
        :param site: site to calculate convection on
        :param output_file: output file name (=> index)
        :param invert: if True, multiplies result by -1
        :return:
        """
        temp = Utils.get_temp_file()
        Utils.cdo.fldmin(input='-selindexbox,{0[0]},{0[1]},{0[2]},{0[3]} {1}'.format(site, input_file),
                         output=temp)
        if invert:
            Utils.cdo.mulc(-1, input=temp, output=output_file)
            os.remove(temp)
        else:
            shutil.move(temp, output_file)


def main():
    try:
        Circulation.psi("ORCA1_MM_19601101_19610228_grid_U.nc", "ORCA1_MM_19601101_19610228_grid_V.nc", "out_psi.nc")
        Circulation.gyres("/scratch/Earth/jvegas/compsstest/out_psi.nc",
                          Grid.ECEARTH_2_3_O1L42, "out_gyres.nc")
        Circulation.moc("ORCA1_MM_19601101_19610228_grid_V.nc", "out_moc.nc")
        Circulation.max_moc("out_moc.nc", 40, 80, 'out_max<_moc.nc', 1000, 2000)
        Circulation.area_moc("out_moc.nc", 40, 80, 'out_area_moc.nc', 1000, 2000)
    except NCOException as ex:
        Log.error('Exception {0}: stderr: {1}, stdout: {2}',  ex.msg, ex.stderr, ex.stdout)
    Utils.clean()


if __name__ == "__main__":
    main()
