from earthdiagnostics import Utils, cdftools
from autosubmit.config.log import Log
import os
import shutil


class Salinity:
    @staticmethod
    def vertical_mean(input_file, output_file, upper, lower):
        """
        Vertically averaged salt content
        Created in February 2012      Author : vguemas@ic3.cat

        :param input_file: input grid_T file name
        :param upper: upper depth of the layer (in meters)
        :type upper: int
        :param lower: lower depth of the layer (in meters)
        :type lower: int
        :param output_file: output file name  (=> 2D)
        """
        cdo = Utils.cdo
        nco = Utils.nco

        temp = Utils.get_temp_file()
        temp2 = Utils.get_temp_file()
        Log.info('Calculating vertical mean between {0} and {1}', upper, lower)
        Log.debug('Input file: {0}', input_file)
        Log.debug('Output file: {0}', output_file)
        # test on cdo version: if >1.5.6, remove valid_min/max attributes to avoid values
        # out of that range to be replaced by NaN
        #
        if cdo.version() > u'1.5.6':
            nco.ncatted(input=input_file, output=temp, options='-O -a valid_max,lev,d,,')
            nco.ncatted(input=temp, output=temp, options='-O -a valid_min,lev,d,,')
        else:
            shutil.copy(input_file, temp)

        cdftools.run('cdfvertmean', input=temp, output=temp2, options=['so', 'T', str(upper), str(lower),
                                                                       '-full', '-debug'])
        os.remove(temp)
        Utils.rename_variable(temp2, 'so_vert_mean', 'vertmeansal')
        Utils.setminmax(temp2, ['vertmeansal'])
        shutil.move(temp2, output_file)
        Log.result('Finished!')

    @staticmethod
    def heat_sal_mxl(input_file, output_file):
        """
        Compute mixed layer heat and salt content

        Created in February 2012      Author : vguemas@ic3.cat

        :param input_file: input grid_T file name
        :param output_file: output file name  (=> 2D x-y )
        :return:
        """

        cdo = Utils.cdo
        nco = Utils.nco

        temp = Utils.get_temp_file()
        temp2 = Utils.get_temp_file()

        handler = cdo.openCdf(input_file)

        if 'time' in handler.variables:
            nco.ncrename(input=input_file, output=temp, options='-O -d time,time_counter -v time,time_counter')
        else:
            shutil.copy(input_file, temp)

        if 'somxl010' not in handler.variables:
            Log.info('Computing mixed layer')
            cdftools.run('cdfmxl', input=temp, output=temp2)
            nco.ncrename(input=temp2, output=temp2, options='-d time_counter,time')
            nco.ncks(input=temp2, output=temp, options='-A')
            os.remove(temp2)

        Log.info('Computing heat content')
        cdftools.run('cdfmxlheatc', input=temp, options=['-full'])

        if 'vosaline' in handler.variables:
            Log.info('Computing salt content')
            cdftools.run('cdfmxlsaltc', input=temp, options=['-full'])
            nco.ncks(input='mxlsaltc.nc', output='mxlheatc.nc', options='-A')
            os.remove('mxlsaltc.nc')
        shutil.move('mxlheatc.nc', temp)

        nco.ncrename(input=temp, output=temp, options='-v .time_counter,time -d .time_counter,time')

        handler = cdo.openCdf(temp)
        variables = ['somxlheatc']
        if 'somxlsaltc' in handler.variables:
            variables.append('somxlsaltc')

        Utils.setminmax(temp, variables)
        shutil.move(temp, output_file)
        Log.result('Finished!')


def main():
    Salinity.vertical_mean("ORCA1_MM_19601101_19610228_grid_T.nc", "out.nc", 300, 5400)
    Salinity.heat_sal_mxl("ORCA1_MM_19601101_19610228_grid_T.nc", "out.nc")


if __name__ == "__main__":
    main()
